/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 04/09/22.
//

#include "robot/programmer.hpp"

namespace demo::robot {

	Timeline::Timeline(Program& program) {
		m_tracks.push_back(std::ref(program));
	}

	void Timeline::update(float deltaTime) {
		auto currSize = size();

		float trackHeight = m_tracks.size() * 32.0F;
		std::size_t widestTrack = 0;
		for ( auto& track : m_tracks ) {
			widestTrack = std::max(widestTrack, track.get().size());
		}

		float instructionWidth = 32 * widestTrack;
		if ( currSize.x < instructionWidth || currSize.y < trackHeight ) {
			size( topLeft(), {instructionWidth, trackHeight} );
		}
	}

	void Timeline::render(fggl::gfx::Paint &paint) {
		fggl::gui::Panel::render(paint);

		renderInstructions(paint);
	}

	void Timeline::renderInstructions(fggl::gfx::Paint& paint) {
		const auto size = this->size();
		const fggl::math::vec2f barExtents{16, 16};

		for ( auto track=0U; track < m_tracks.size(); ++track) {
			auto& trackRef = m_tracks[track].get();

			for (auto i = 0U; i < trackRef.size(); ++i) {
				auto barCenter = this->topLeft();

				barCenter.x += (i * (barExtents.x * 2) ) + barExtents.x;
				barCenter.y += (track * barExtents.y * 2) + barExtents.y;

				// bar background
				auto colour = fggl::gfx::colours::LIGHT_GRAY;
				auto textColour = fggl::gfx::colours::DARK_SLATE_GRAY;
				if (i % 2 == 0) {
					colour = fggl::gfx::colours::WHITE;
				}

				if (i == trackRef.m_currInstruction && trackRef.playing) {
					colour = fggl::gfx::colours::MIDNIGHT_BLUE;
					textColour = fggl::gfx::colours::LIGHT_GRAY;
				}
				auto rect = fggl::gfx::make_rect(barCenter, barExtents, colour);
				paint.fill(rect);

				// bar instruction
				auto& instruction = trackRef.m_instructions[i];
				paint.text(instruction.name, barCenter, textColour);
			}
		}
	}

}
