/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 23/07/22.
//

#ifndef FGGL_ENTITY_LOADER_SPEC_HPP
#define FGGL_ENTITY_LOADER_SPEC_HPP

#include "fggl/util/guid.hpp"
#include "fggl/entity/loader/serialise.hpp"

#include <map>

namespace fggl::entity {

	using ComponentID = util::GUID;
	using EntityType = util::GUID;
	constexpr EntityType NO_PARENT = util::make_guid("FGGL_NULL_PARENT");

	struct ComponentSpec {

		template<typename T>
		T get(const std::string &key, const T &fallback) const {
			return config[key].template as<T>(fallback);
		}

		template<typename T>
		void set(const std::string &key, const T &value) {
			config[key] = value;
		}

		inline bool has(const std::string &key) const {
			return (bool) (config[key]);
		}

		YAML::Node config;
	};

	struct EntitySpec {
		EntityType parent = NO_PARENT;
		std::vector<ComponentID> ordering;
		std::map<ComponentID, ComponentSpec> components;

		inline void addComp(ComponentID cmp, const ComponentSpec& spec) {
			components[cmp] = spec;
			ordering.push_back(cmp);
		}
	};

} // namespace fggl::entity

#endif //FGGL_ENTITY_LOADER_SPEC_HPP
