/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 19/09/22.
//

#ifndef FGGL_PLATFORM_FALLBACK_FILE_HPP
#define FGGL_PLATFORM_FALLBACK_FILE_HPP

#include <cstdio>
#include <cassert>

namespace fggl::platform {

	class File {
		public:
			inline File(FILE* filePtr) : m_handle(filePtr) {
				assert(filePtr != nullptr);
			}

			inline ~File() {
				release();
			}

			template<typename T>
			inline void write(const T* dataPtr) {
				assert( m_handle != nullptr );
				int status = fwrite(dataPtr, sizeof(T), 1, m_handle );
				assert( status == 1);
			}

			template<typename T>
			inline void writeArr(const T* dataPtr, std::size_t numElms) {
				assert( m_handle != nullptr);
				int status = fwrite(dataPtr, sizeof(T), numElms, m_handle );
				assert( status == 1);
			}

		private:
			std::FILE* m_handle;

			inline void release() {
				if ( m_handle != NULL) {
					fclose(m_handle);
				}
			}
	};

} // namespace fggl::platform

#endif //FGGL_PLATFORM_FALLBACK_FILE_HPP
