/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_UTIL_CHRONO_HPP
#define FGGL_UTIL_CHRONO_HPP

namespace fggl::util {

	class Timer {
			using time_t = unsigned long;
		public:
			inline void frequency(float freq) {
				m_freq = freq;
			}

			inline void setup(time_t epoch) {
				m_first = epoch;
				m_last = m_first;
				m_curr = m_first;
			}

			inline void tick(time_t time) {
				m_last = m_curr;
				m_curr = time;
			}

			[[nodiscard]]
			inline float delta() const {
				return (m_curr - m_last) / m_freq;
			}

		private:
			float m_freq;
			time_t m_first;
			time_t m_last;
			time_t m_curr;
	};
};

#endif
